unit f_guide;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, FormatLabel, Buttons, ComCtrls,
  URLLabel, ApplicationHook, AppGuide_types, ExtCtrls;

type
  TGuide = class(TForm)
    pClient: TPanel;
    pMenu: TPanel;
    lblStep: TFormatLabel;
    btPrev: TURLLabel;
    btNext: TURLLabel;
    FormatLabel3: TFormatLabel;
    sb: TScrollBox;
    lblText: TFormatLabel;
    FormatLabel1: TFormatLabel;
    btContent: TURLLabel;
    FormatLabel5: TFormatLabel;
    btBack: TURLLabel;
    AppHook: TApplicationHook;
    pc: TPageControl;
    tsEmpty: TTabSheet;
    tsTip: TTabSheet;
    btHide: TURLLabel;
    cbOnTop: TCheckBox;
    cbDoNotShow: TCheckBox;
    pClose: TPanel;
    btClose: TBitBtn;
    Timer: TTimer;
    procedure cbOnTopClick(Sender: TObject);
    procedure sbResize(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure AppHookMessage(Sender: TObject; var Message: TMessage;
      var Handled: Boolean);
    procedure lblStepGetParameter(Sender: TObject; var ParamStr: String);
    procedure btPrevClick(Sender: TObject);
    procedure btNextClick(Sender: TObject);
    procedure lblTextAnchorClick(Sender: TObject; LinkName: String);
    procedure btContentClick(Sender: TObject);
    procedure btBackClick(Sender: TObject);
    procedure btHideClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure cbDoNotShowClick(Sender: TObject);
  private
    { Private declarations }
    FMemRec: PMemRec;
    RosiMsg : UINT;
    FSharedMem: TSharedMem;
    L: TStrings;
    Hist: TStrings;

    procedure ShowText(s: string);
    procedure AddLine(s: string);
    procedure SetTopic(Name: string; Step: integer);
    procedure ProcInit(Param: integer);
    procedure LoadFile;
    procedure LoadTopic;
    procedure RefBtn;
    procedure SetWinState;
  public
    { Public declarations }
    function IsLinked: boolean;
  end;

var
  Guide: TGuide;

implementation

uses rstring, rtool;

{$R *.dfm}

procedure TGuide.FormCreate(Sender: TObject);
begin
  RosiMsg:=RegisterWindowMessage(msgName);
  FSharedMem:=TSharedMem.Create(Application.Title, SHAREMEM_SIZE);
  FMemRec:=FSharedMem.Buffer;
  FMemRec.GuideHandle:=Application.Handle;
  L:=TStringList.Create;
  Hist:=TStringList.Create;
  ProcInit(1);
  if FMemRec.AppHandle=0 then
  begin
    FMemRec.ShowButtons:=true;
    pMenu.Visible:=true;
    if ParamCount>0 then
    begin
      FMemRec.TXTFile:=ParamStr(1);
      FMemRec.TopicName:=ContentName;
    end
    else
    begin
      FMemRec.TXTFile:=ChangeFileExt(Application.ExeName,'.ini');
      FMemRec.TopicName:=ContentName;
    end;
  end;
  AppHook.Activate;
  LoadFile;
  LoadTopic;
end;

procedure TGuide.FormShow(Sender: TObject);
begin
  if IsLinked then ShowWindow(Application.Handle,SW_HIDE);
end;

procedure TGuide.FormActivate(Sender: TObject);
begin
  ;
end;

procedure TGuide.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FMemRec.GuideHandle:=0;
  FMemRec.WinPosTop:=Top;
  FMemRec.WinPosLeft:=Left;
  FMemRec.WinWidth:=Width;
  FMemRec.WinHeight:=Height;
  FMemRec.WinOnTop:=cbOnTop.Checked;
  //FMemRec.DoNotShow:=cbDoNotShow.Checked;
  if FMemRec.AppHandle<>0 then
    PostMessage(FMemRec.AppHandle,RosiMsg,msgClose,0);
end;

procedure TGuide.FormDestroy(Sender: TObject);
begin
  FSharedMem.Free;
  L.Free;
  Hist.Free;
end;

procedure TGuide.sbResize(Sender: TObject);
begin
  lblText.AutoSize:=false;
  lblText.AutoSize:=true;
end;

procedure TGuide.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
var i: integer;
begin
  if PtInRect(Rect(Left,Top,Left+Width,Top+Height),MousePos) then
  begin
    Handled:=true;
    i:=40;
    if WheelDelta<0 then i:=-i;
    sb.VertScrollBar.Position:=sb.VertScrollBar.Position-i;
  end;
end;

function TGuide.IsLinked: boolean;
begin
  Result:=FMemRec.AppHandle<>0;
end;

procedure TGuide.SetWinState;
begin
  if cbOnTop.Checked then
  begin
    if IsLinked then ShowWindow(Application.Handle,SW_HIDE);
  end
  else
  begin
    ShowWindow(Application.Handle,SW_SHOW);
  end;
end;

procedure TGuide.btCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TGuide.cbOnTopClick(Sender: TObject);
begin
  if cbOnTop.Checked then Self.FormStyle:=fsStayOnTop
                     else Self.FormStyle:=fsNormal;
  SetWinState;
end;

procedure TGuide.AppHookMessage(Sender: TObject; var Message: TMessage;
  var Handled: Boolean);
begin
  if Message.Msg=RosiMsg then
  begin
    case Message.WParam of
      msgClose: Close;
      msgShow: begin
                 Show;
                 SetWinState;
               end;
      msgHide: begin
                 Hide;
                 Timer.Enabled:=false;
               end;
      msgInit: ProcInit(Message.LParam);
      msgLoadFile: begin Hist.Clear; LoadFile; end;
      msgLoadTopic: begin Hist.Clear; LoadTopic; end;
    end;
    Handled := true;
  end;
end;

procedure TGuide.ProcInit(Param: integer);
begin
  if FMemRec.WinSetPos then
    SetBounds(FMemRec.WinPosLeft,FMemRec.WinPosTop,
              FMemRec.WinWidth,FMemRec.WinHeight)
  else
    if Param<>0 then
    begin
      Width:=Screen.Width div 3;
      Height:=Screen.Height div 3;
      SetBounds(Screen.Width-Width,0,Width,Height);
    end;
  cbOnTop.Checked:=FMemRec.WinOnTop;
  pMenu.Visible:=FMemRec.ShowButtons;
end;

procedure TGuide.LoadFile;
begin
  btContent.Enabled:=false;
  try
    L.LoadFromFile(FMemRec.TXTFile);
    btContent.Enabled:=L.IndexOf('['+ContentName+']')>=0;
    if FMemRec.TopicName='' then
    begin
      FMemRec.TopicName:=ContentName;
      LoadTopic;
    end;
  except
    ShowText(Format(sFileNotFound,[FMemRec.TXTFile]));
  end;
end;

procedure TGuide.ShowText(s: string);
begin
  lblText.Caption:='';
  lblText.CaptionStrings.Text:=s;
end;

procedure TGuide.AddLine(s: string);
begin
  lblText.CaptionStrings.Add(s);
end;

procedure TGuide.SetTopic(Name: string; Step: integer);
begin
  if (Name=FMemRec.TopicName) and (Step=FMemRec.Step) then Exit;
  Hist.AddObject(FMemRec.TopicName,TObject(FMemRec.Step));
  FMemRec.TopicName:=Name;
  FMemRec.Step:=Step;
  LoadTopic;
end;

procedure TGuide.LoadTopic;
var t,s,pline: string;
    i,actStep,maxStep: integer;
begin
  if L.Count=0 then Exit;
  // Init all
  cbDoNotShow.Enabled:=(FMemRec.ShowParam AND 1)<>0;
  cbDoNotShow.Checked:=FMemRec.DoNotShow;
  lblText.CaptionStrings.Clear;
  pline:='';
  if FMemRec.Step=0 then FMemRec.Step:=1;
  actStep:=0;
  maxStep:=1;
  // find start point
  t:='['+FMemRec.TopicName+']';
  i:=L.IndexOf(t);
  if i<0 then
  begin
    FMemRec.Step:=0;
    FMemRec.MaxStep:=0;
    ShowText(Format(sGiudeTopicErr,[FMemRec.TopicName]));
  end
  else
  begin
    repeat
      Inc(i);
      s:=Trim(L[i]);
      // next topic? = end of my topic
      if (copy(s,1,1)='[') and (copy(s,Length(s),1)=']') then Break;
      // step ?
      if (copy(s,1,1)='#') and (copy(s,Length(s),1)='#') then
        try
          actStep:=StrToInt(copy(s,2,Length(s)-2));
          if actStep>maxStep then maxStep:=actStep;
        except
        end
      else
      if (actStep=0) or (actStep=FMemRec.Step) then
      begin
        if (copy(s,Length(s),1)='\') then
          pline:=pline+Trim(copy(s,1,Length(s)-1))+' '
        else
        begin
          AddLine(pline+s);
          pline:='';
        end;
      end;
    until i>=L.Count-1;
    FMemRec.MaxStep:=maxStep;
  end;
  RefBtn;
end;

procedure TGuide.RefBtn;
begin
  lblStep.Invalidate;
  btPrev.Enabled:=FMemRec.Step>1;
  btNext.Enabled:=FMemRec.Step<FMemRec.MaxStep;
  btBack.Enabled:=Hist.Count>0;
end;

procedure TGuide.lblStepGetParameter(Sender: TObject; var ParamStr: String);
begin
  try
    if ParamStr='S1' then ParamStr:=IntToStr(FMemRec.Step);
    if ParamStr='S2' then ParamStr:=IntToStr(FMemRec.MaxStep);
  except
  end;
end;

procedure TGuide.cbDoNotShowClick(Sender: TObject);
begin
  FMemRec.DoNotShow:=cbDoNotShow.Checked;
end;

procedure TGuide.btPrevClick(Sender: TObject);
begin
  if FMemRec.Step<=1 then Exit;
  FMemRec.Step:=FMemRec.Step-1;
  LoadTopic;
end;

procedure TGuide.btNextClick(Sender: TObject);
begin
  if FMemRec.Step>=FMemRec.MaxStep then Exit;
  FMemRec.Step:=FMemRec.Step+1;
  LoadTopic;
end;

procedure TGuide.btContentClick(Sender: TObject);
begin
  if not btContent.Enabled then Exit;
  SetTopic(ContentName,1);
end;

procedure TGuide.btBackClick(Sender: TObject);
begin
  if Hist.Count<=0 then Exit;
  FMemRec.TopicName:=Hist[Hist.Count-1];
  FMemRec.Step:=Integer(Hist.Objects[Hist.Count-1]);
  Hist.Delete(Hist.Count-1);
  LoadTopic;
end;

procedure TGuide.lblTextAnchorClick(Sender: TObject; LinkName: String);
var Name: string;
    Step: integer;
begin
  if copy(LinkName,1,1)='@' then
  begin
    FMemRec.Command:=copy(LinkName,2,Length(LinkName));
    if FMemRec.AppHandle<>0 then
      PostMessage(FMemRec.AppHandle,RosiMsg,msgCommand,0);
  end
  else
  begin
    Name:=FindStrPart(LinkName,'#',1,false);
    try
      Step:=StrToInt(FindStrPart(LinkName,'#',2,false));
    except
      Step:=1;
    end;
    SetTopic(Name,Step);
  end;
end;

procedure TGuide.btHideClick(Sender: TObject);
begin
  Hide;
  Timer.Enabled:=true;
end;

procedure TGuide.TimerTimer(Sender: TObject);
begin
  Timer.Enabled:=false;
  Show;
end;

end.
