unit AppGuide_types;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms;

const
  msgName='RosiGuideMsg';
  msgClose=1;
  msgShow=2;
  msgHide=3;
  msgInit=10;
  msgLoadFile=11;
  msgLoadTopic=13;
  msgCommand=20;

  ContentName='Content';
  
  SHAREMEM_SIZE=1000;

type
  TMemRec=record
    AppHandle: HWND;
    GuideHandle: HWND;
    WinSetPos: boolean;
    WinPosTop: integer;
    WinPosLeft: integer;
    WinWidth: integer;
    WinHeight: integer;
    WinOnTop: boolean;
    DoNotShow: boolean;
    ShowButtons: boolean;
    TXTFile: string[250];
    TopicName: string[50];
    Step: integer;
    MaxStep: integer;
    ShowParam: integer;
    Command: string[50];
  end;
  PMemRec=^TMemRec;

  THandledObject = class(TObject)
  protected
    FHandle: THandle;
  public
    destructor Destroy; override;
    property Handle: THandle read FHandle;
  end;

  TSharedMem = class(THandledObject)
  private
    FName: string;
    FSize: Integer;
    FCreated: Boolean;
    FFileView: Pointer;
  public
    constructor Create(const Name: string; Size: Integer);
    destructor Destroy; override;
    property Name: string read FName;
    property Size: Integer read FSize;
    property Buffer: Pointer read FFileView;
    property Created: Boolean read FCreated;
  end;


implementation

procedure Error(const Msg: string);
begin
  raise Exception.Create(Msg);
end;

destructor THandledObject.Destroy;
begin
  if FHandle <> 0 then
    CloseHandle(FHandle);
end;

constructor TSharedMem.Create(const Name: string; Size: Integer);
begin
  try
    FName := Name;
    FSize := Size;
    { CreateFileMapping, when called with $FFFFFFFF for the hanlde value,
      creates a region of shared memory }
    FHandle := CreateFileMapping($FFFFFFFF, nil, PAGE_READWRITE, 0,
        Size, PChar(Name));
    if FHandle = 0 then abort;
    FCreated := GetLastError = 0;
    { We still need to map a pointer to the handle of the shared memory region }
    FFileView := MapViewOfFile(FHandle, FILE_MAP_WRITE, 0, 0, Size);
    if FFileView = nil then abort;
  except
    Error(Format('Error creating shared memory %s (%d)', [Name, GetLastError]));
  end;
end;

destructor TSharedMem.Destroy;
begin
  if FFileView <> nil then
    UnmapViewOfFile(FFileView);
  inherited Destroy;
end;

end.
